@@ -182,29 +182,14 @@ class Feed(object):
         - dates[-1]: ditto
 
         If ``dates is None``, then return ``None``.
-
-        NOTES:
-
-        Takes about 0.15 minutes on the SEQ feed for 7 dates.
         """
         if not dates:
             return
 
         f = self.trips
-
-        # Time the function call
-        t1 = dt.datetime.now()
-        print(t1, 'Getting activity for {!s} trips over {!s} dates...'.format(
-          f.shape[0], len(dates)))
-
         for date in dates:
             f[date] = f['trip_id'].map(lambda trip: 
               int(self.is_active_trip(trip, date)))
-
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished trips activity in %.2f min' % minutes)    
-
         return f[['trip_id', 'direction_id', 'route_id'] + dates]
 
     def get_trips_stats(self):
@@ -218,25 +203,21 @@ class Feed(object):
         - end_time: last departure time of the trip
         - start_stop_id: stop ID of the first stop of the trip 
         - end_stop_id: stop ID of the last stop of the trip
-        - duration: duration of the trip (seconds)
-        - distance: distance of the trip (meters); contains all ``np.nan``
+        - duration: duration of the trip in hours
+        - distance: distance of the trip in kilometers; contains all ``np.nan``
           entries if ``self.shapes is None``
 
         NOTES:
 
-        Takes about 2.4 minutes on the SEQ feed.
+        Takes about 1 minute on the Portland feed.
         """
         trips = self.trips
         stop_times = self.stop_times
-
-        t1 = dt.datetime.now()
         num_trips = trips.shape[0]
-        print(t1, 'Creating trip stats for %s trips...' % num_trips)
-
+        
         # Initialize data frame. Base it on trips.txt.
         stats = trips[['route_id', 'trip_id', 'direction_id']]
 
-
         # Compute start time, end time, duration
         f = pd.merge(trips, stop_times)
         # Convert departure times to seconds past midnight, 
@@ -245,7 +226,7 @@ class Feed(object):
           lambda x: utils.seconds_to_timestr(x, inverse=True))
         f = f.groupby('trip_id')
         g = f['departure_time'].agg({'start_time': np.min, 'end_time': np.max})
-        g['duration'] = g['end_time'] - g['start_time']
+        g['duration'] = (g['end_time'] - g['start_time'])/3600
 
         # Compute start stop and end stop
         def start_stop(group):
@@ -263,8 +244,8 @@ class Feed(object):
         g[['start_time', 'end_time']] = g[['start_time', 'end_time']].\
           applymap(lambda x: utils.seconds_to_timestr(int(x)))
 
-        # Compute trip distance, which is more involved and requires
-        # shapes
+        # Compute trip distance (in meters), 
+        # which is more involved and requires self.shapes
         if self.shapes is not None:
             g['shape_id'] = f['shape_id'].first()
             linestring_by_shape = self.get_linestring_by_shape()
@@ -298,13 +279,12 @@ class Feed(object):
         else:
             g['distance'] = np.nan
 
+        # Convert distance from meters to kilometers
+        g['distance'] /= 1000
+
         stats = pd.merge(stats, g.reset_index())
         stats.sort('route_id')
 
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished trips stats in %.2f min' % minutes)    
-    
         return stats
 
     def get_linestring_by_shape(self):
@@ -437,6 +417,7 @@ class Feed(object):
         Return a Pandas data frame with the following columns:
 
         - stop_id
+        - direction_id
         - mean_daily_num_vehicles: mean daily number of vehicles visiting stop 
         - max_headway: maximum of the durations (in seconds) between 
           vehicle departures at the stop between 07:00 and 19:00 
@@ -449,21 +430,14 @@ class Feed(object):
         - max_end_time: latest departure time of a vehicle from this stop
           over the given date range
 
-        If ``split_directions == True``, then add an extra column
-
-        - direction_id: 0 or 1,
-
-        and separate the stats above by the direction ID of the trips
-        visiting each stop.
-        So each stop_id will have two rows.
+        If ``split_directions == False``, then compute each stop's stats
+        using vehicles visiting it from both directions.
 
         NOTES:
 
-        Takes about 0.9 minutes for the SEQ feed.
+        Takes about 0.73 minutes on the Portland feed given the first
+        five weekdays of the feed.
         """
-        t1 = dt.datetime.now()
-        print(t1, 'Calculating stops stats...')
-
         # Get active trips and merge with stop times
         trips_activity = self.get_trips_activity(dates)
         ta = trips_activity[trips_activity[dates].sum(axis=1) > 0]
@@ -512,108 +486,57 @@ class Feed(object):
             df.index.name = 'foo'
             return df
 
-        # Compute stats for each stop by direction
-        def get_stop_stats_by_direction(group):
-            # Operate on the group of all stop times for an individual stop
-            directions = [0, 1]
-            headways_by_dir = {d: [] for d in directions}
-            num_vehicles_by_dir = {d: 0 for d in directions}
-            max_headway_by_dir = {d: np.nan for d in directions}
-            mean_headway_by_dir = {d: np.nan for d in directions}
-            mean_daily_num_vehicles_by_dir = {d: 0 for d in directions}
-            min_start_time_by_dir = {d: np.nan for d in directions}
-            max_end_time_by_dir = {d: np.nan for d in directions}  
-            group_by_dir = {d: None for d in directions}          
-            for d in directions:
-                group_by_dir[d] = group[group['direction_id'] == d]
-                g = group_by_dir[d]
-                for date in dates:
-                    dtimes = sorted(g[g[date] > 0]['departure_time'].\
-                      values)
-                    num_vehicles_by_dir[d] += len(dtimes)
-                    dtimes = [dtime for dtime in dtimes 
-                      if 7*3600 <= dtime <= 19*3600]
-                    headways_by_dir[d].extend(
-                      [dtimes[i + 1] - dtimes[i] 
-                      for i in range(len(dtimes) - 1)])
-                if headways_by_dir[d]:
-                    max_headway_by_dir[d] = np.max(
-                      headways_by_dir[d])
-                    mean_headway_by_dir[d] = round(np.mean(
-                      headways_by_dir[d]))
-                mean_daily_num_vehicles_by_dir[d] =\
-                  num_vehicles_by_dir[d]/len(dates)
-                min_start_time_by_dir[d] = g['departure_time'].min()
-                max_end_time_by_dir[d] = g['departure_time'].max()
-            df = pd.DataFrame([[
-              d,
-              min_start_time_by_dir[d], 
-              max_end_time_by_dir[d], 
-              mean_daily_num_vehicles_by_dir[d], 
-              max_headway_by_dir[d], 
-              mean_headway_by_dir[d],
-              ] for d in directions], 
-              columns=[
-              'direction_id',
-              'min_start_time', 
-              'max_end_time', 
-              'mean_daily_num_vehicles', 
-              'max_headway', 
-              'mean_headway',
-              ])
-            df.index.name = 'foo'
-            return df
-
         if split_directions:
-            result = f.groupby('stop_id').apply(get_stop_stats_by_direction).\
-            reset_index()
+            g = f.groupby(['stop_id', 'direction_id'])
         else:
-            result = f.groupby('stop_id').apply(get_stop_stats).reset_index()
+            g = f.groupby('stop_id')
+
+        result = g.apply(get_stop_stats).reset_index()
+
         # Convert start and end times to time strings
         result[['min_start_time', 'max_end_time']] =\
           result[['min_start_time', 'max_end_time']].applymap(
           lambda x: utils.seconds_to_timestr(x))
         del result['foo']
 
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished stops stats in %.2f min' % minutes)    
-
         return result
 
-    def get_stops_time_series(self, dates):
+    def get_stops_time_series(self, dates, split_directions=True,
+      freq='5Min'):
         """
-        Return the following time series of stops stats:
+        Return a time series version of the following stops stats
+        for the given dates:
         
         - mean daily number of vehicles by stop ID
 
-        The time series is a Pandas data frame over a 24-hour 
-        period with minute (period index) frequency (00:00 to 23:59).
-        
-        Return the time series as a value in a dictionary with key
-        'mean_daily_num_vehicles'. 
-        (Outputing a dictionary of a time series instead of simply a 
-        time series matches the structure of ``get_routes_time_series()``
-        and allows for the possibility of adding other stops time series
-        at a later stage of development.)
+        The time series is a Pandas data frame with a period index 
+        for a 24-hour period sampled at the given frequency.
+        The maximum allowable frequency is 1 minute.
+        If multiples dates are given, a generic placeholder date of
+        2001-01-01 is used as the date for the period index.
+        Otherwise, the given date is used.
 
+        The columns of the data frame are hierarchical (multi-index) with
+
+        - top level: name = 'statistic', values = ['mean_daily_num_vehicles']
+        - middle level: name = 'stop_id', values = the active stop IDs
+        - bottom level: name = 'direction_id', values = 0s and 1s
+
+        If ``split_directions == False``, then don't include the bottom level.
+        
         NOTES:
 
-        - To resample the resulting time series use the following methods:
-          for 'mean_daily_num_vehicles' series, use ``how=np.sum``
-        - To remove the placeholder date (2001-1-1) and seconds from any 
-          of the time series f, do ``f.index = [t.time().strftime('%H:%M') 
+        - 'mean_daily_num_vehicles' should be resampled with ``how=np.sum``
+        - To remove the placeholder date (2001-1-1) and seconds from 
+          the time series f, do ``f.index = [t.time().strftime('%H:%M') 
           for t in f.index.to_datetime()]``
-        - Takes about 2 minutes on the SEQ feed.
+        - Takes about 6.15 minutes on the Portland feed given the first
+          five weekdays of the feed.
         """  
         if not dates:
             return 
 
-        t1 = dt.datetime.now()
-        stops = sorted(self.stops['stop_id'].values)
-        num_stops = len(stops)
-        print(t1, 'Creating stops time series for %s stops...' % num_stops)
-
+        num_stops = len(self.stops['stop_id'].values)
         # Get active trips and merge with stop times
         trips_activity = self.get_trips_activity(dates)
         ta = trips_activity[trips_activity[dates].sum(axis=1) > 0]
@@ -633,6 +556,11 @@ class Feed(object):
         rng = pd.period_range(day_start, day_end, freq='Min')
         names = ['mean_daily_num_vehicles']
         series_by_name = {}
+        if split_directions:
+            stops = [str(s) + '-' + d for s in stats['stop_id'].unique()
+              for d in ['0', '1']]
+        else:
+            stops = stats['stop_id'].unique()
         for name in names:
             series_by_name[name] = pd.DataFrame(np.nan, index=rng, 
               columns=stops)
@@ -649,52 +577,76 @@ class Feed(object):
         # Bin each trip according to its departure time at the stop
         i = 0
         f = series_by_name['mean_daily_num_vehicles']
-        for stop, group in stats.groupby('stop_id'):
-            i += 1
-            print("Progress {:2.1%}".format(i/num_stops), end="\r")
-            for index, row in group.iterrows():
-                weight = row['weight']
-                dtime = row['departure_time']
-                # Bin stop time
-                criterion = f.index == dtime
-                g = f.loc[criterion, stop] 
-                # Use fill_value=0 to overwrite NaNs with numbers.
-                # Need to use pd.Series() to get fill_value to work.
-                f.loc[criterion, stop] = g.add(pd.Series(
-                  weight, index=g.index), fill_value=0)
+        if split_directions:
+            for (stop, direction), group in stats.groupby(
+              ['stop_id', 'direction_id']):
+                i += 1
+                print("Progress {:2.1%}".format(i/(2*num_stops)), end="\r")
+                stop = str(stop) + '-' + str(direction)
+                for index, row in group.iterrows():
+                    weight = row['weight']
+                    dtime = row['departure_time']
+                    # Bin stop time
+                    criterion = f.index == dtime
+                    g = f.loc[criterion, stop] 
+                    # Use fill_value=0 to overwrite NaNs with numbers.
+                    # Need to use pd.Series() to get fill_value to work.
+                    f.loc[criterion, stop] = g.add(pd.Series(
+                      weight, index=g.index), fill_value=0)
+        else:
+            for stop, group in stats.groupby('stop_id'):
+                i += 1
+                print("Progress {:2.1%}".format(i/num_stops), end="\r")
+                for index, row in group.iterrows():
+                    weight = row['weight']
+                    dtime = row['departure_time']
+                    # Bin stop time
+                    criterion = f.index == dtime
+                    g = f.loc[criterion, stop] 
+                    # Use fill_value=0 to overwrite NaNs with numbers.
+                    # Need to use pd.Series() to get fill_value to work.
+                    f.loc[criterion, stop] = g.add(pd.Series(
+                      weight, index=g.index), fill_value=0)
       
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished stops time series in %.2f min' % minutes)    
-
-        return series_by_name
+        f = utils.combine_time_series(series_by_name, kind='stop')
+        return utils.downsample(f, freq=freq)
 
     def get_stops_in_stations(self):
         """
-        Assuming this feed has station data, that is, 'location_type' and
+        If this feed has station data, that is, 'location_type' and
         'parent_station' columns in ``self.stops``, then return a Pandas
         data frame that has the same columns as ``self.stops``
         but only includes stops with parent stations, that is, stops with
         location type 0 or blank and nonblank parent station.
+        Otherwise, return ``None``.
         """
         f = self.stops
-        return f[(f['location_type'] != 1) & (f['parent_station'].notnull())]
+        result = f[(f['location_type'] != 1) & (f['parent_station'].notnull())]
+        if result.empty:
+            return
+        return result
 
-    def get_stations_stats(self, dates, split_directions=False):
+    def get_stations_stats(self, dates, split_directions=True):
         """
-        Assuming this feed has station data, that is, 'location_type' and
+        If this feed has station data, that is, 'location_type' and
         'parent_station' columns in ``self.stops``, then compute
-        the same stats that ``self.get_stops_stats()`` does, but format_str
+        the same stats that ``self.get_stops_stats()`` does, but for
         stations.
-        """
-        t1 = dt.datetime.now()
-        print(t1, 'Calculating stops stats...')
+        Otherwise, return ``None``.
 
+        NOTES:
+
+        Takes about 0.2 minutes on the Portland feed given the first
+        five weekdays of the feed.
+        """
         # Get stop times of active trips that visit stops in stations
         stop_times = self.stop_times
         trips_activity = self.get_trips_activity(dates)
         ta = trips_activity[trips_activity[dates].sum(axis=1) > 0]
         sis = self.get_stops_in_stations()
+        if sis is None:
+            return
+
         f = pd.merge(stop_times, ta)
         f = pd.merge(f, sis)
 
@@ -741,64 +693,12 @@ class Feed(object):
             df.index.name = 'foo'
             return df
 
-        # Compute stats for each station by direction
-        def get_station_stats_by_direction(group):
-            # Operate on the group of all stop times for an individual stop
-            directions = [0, 1]
-            headways_by_dir = {d: [] for d in directions}
-            num_vehicles_by_dir = {d: 0 for d in directions}
-            max_headway_by_dir = {d: np.nan for d in directions}
-            mean_headway_by_dir = {d: np.nan for d in directions}
-            mean_daily_num_vehicles_by_dir = {d: 0 for d in directions}
-            min_start_time_by_dir = {d: np.nan for d in directions}
-            max_end_time_by_dir = {d: np.nan for d in directions}  
-            group_by_dir = {d: None for d in directions}          
-            for d in directions:
-                group_by_dir[d] = group[group['direction_id'] == d]
-                g = group_by_dir[d]
-                for date in dates:
-                    dtimes = sorted(g[g[date] > 0]['departure_time'].\
-                      values)
-                    num_vehicles_by_dir[d] += len(dtimes)
-                    dtimes = [dtime for dtime in dtimes 
-                      if 7*3600 <= dtime <= 19*3600]
-                    headways_by_dir[d].extend(
-                      [dtimes[i + 1] - dtimes[i] 
-                      for i in range(len(dtimes) - 1)])
-                if headways_by_dir[d]:
-                    max_headway_by_dir[d] = np.max(
-                      headways_by_dir[d])
-                    mean_headway_by_dir[d] = round(np.mean(
-                      headways_by_dir[d]))
-                mean_daily_num_vehicles_by_dir[d] =\
-                  num_vehicles_by_dir[d]/len(dates)
-                min_start_time_by_dir[d] = g['departure_time'].min()
-                max_end_time_by_dir[d] = g['departure_time'].max()
-            df = pd.DataFrame([[
-              d,
-              min_start_time_by_dir[d], 
-              max_end_time_by_dir[d], 
-              mean_daily_num_vehicles_by_dir[d], 
-              max_headway_by_dir[d], 
-              mean_headway_by_dir[d],
-              ] for d in directions], 
-              columns=[
-              'direction_id',
-              'min_start_time', 
-              'max_end_time', 
-              'mean_daily_num_vehicles', 
-              'max_headway', 
-              'mean_headway',
-              ])
-            df.index.name = 'foo'
-            return df
-
         if split_directions:
-            result = f.groupby('parent_station').apply(
-              get_station_stats_by_direction).reset_index()
+            g = f.groupby(['parent_station', 'direction_id'])
         else:
-            result = f.groupby('parent_station').apply(
-              get_station_stats).reset_index()
+            g = f.groupby('parent_station')
+
+        result = g.apply(get_station_stats).reset_index()
 
         # Convert start and end times to time strings
         result[['min_start_time', 'max_end_time']] =\
@@ -806,43 +706,19 @@ class Feed(object):
           lambda x: utils.seconds_to_timestr(x))
         del result['foo']
 
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished stops stats in %.2f min' % minutes)    
-
         return result
 
-    # def get_stop_timetable(self, stop, date):
-    #     """
-    #     Given a stop (stop_id) and a date (datetime.date object),
-    #     return a Pandas data frame describing the time table for the
-    #     given stop on the given date with the columns
-
-    #     - arrival_time
-    #     - departure_time
-    #     - trip_id
-    #     - route_id
-
-    #     and ordered by arrival time.
-    #     """
-    #     # Get active trips and merge with stop times
-    #     trips_activity = self.get_trips_activity([date])
-    #     ta = trips_activity[trips_activity[date] > 0]
-    #     stop_times = self.stop_times
-    #     f = pd.merge(ta, stop_times)
-    #     return f[f['stop_id'] == stop][['arrival_time', 'departure_time', 
-    #       'trip_id', 'route_id']].sort('arrival_time').reset_index(drop=True)
-
     def get_routes_stats(self, trips_stats, dates, split_directions=True):
         """
         Take ``trips_stats``, which is the output of 
         ``self.get_trips_stats()``, and use it to calculate stats for 
-        all the routs in this feed averaged over the given dates 
+        all the routes active on at least one day of the given dates
         (list of ``datetime.date`` objects). 
         
         Return a Pandas data frame with the following columns
 
-        - route_id: route ID
+        - route_id
+        - direction_id
         - mean_daily_num_trips
         - min_start_time: start time of the earliest active trip on 
           the route
@@ -851,29 +727,25 @@ class Feed(object):
           trip starts on the route between 07:00 and 19:00 on the given dates
         - mean_headway: mean of the durations (in seconds) between 
           trip starts on the route between 07:00 and 19:00 on the given dates
-        - mean_daily_duration: in seconds
-        - mean_daily_distance: in meters; contains all ``np.nan`` entries if 
-          ``self.shapes is None``  
+        - mean_daily_duration: in hours
+        - mean_daily_distance: in kilometers; contains all ``np.nan`` entries
+          if ``self.shapes is None``  
+        - mean_daily_speed: in kilometers per hour
 
-        If ``split_directions == True``, then add an extra column
-
-        - direction_id: 0 or 1,
-
-        and separate the stats above by the direction ID of the trips
-        on each route.
+        If ``split_directions == False``, then remove the direction_id column
+        and compute each route's stats based using its trips running in both 
+        directions. 
+        Note that this will give bidirectional headway stats, which most folks
+        don't find useful.
 
         NOTES:
 
-        Takes about 0.2 minute on the SEQ feed for 5 dates.
+        Takes about 0.2 minutes on the Portland feed given the first
+        five weekdays of the feed.
         """
         if not dates:
             return 
 
-        # Time the function call
-        t1 = dt.datetime.now()
-        print(t1, 'Creating routes stats for {!s} routes...'.format(
-          self.routes.shape[0]))
-
         # Merge trips stats with trips activity, 
         # assign a weight to each trip equal to the fraction of days in 
         # dates for which it is active, and and drop 0-weight trips
@@ -927,128 +799,83 @@ class Feed(object):
             df.index.name = 'foo'
             return df
 
-        def get_route_stats_by_direction(group):
-            # Take this group of all trips stats for a single route
-            # and compute route-level stats.
-            directions = [0, 1]
-            headways_by_dir = {d: [] for d in directions}
-            num_vehicles_by_dir = {d: 0 for d in directions}
-            max_headway_by_dir = {d: np.nan for d in directions}
-            mean_headway_by_dir = {d: np.nan for d in directions}
-            mean_daily_num_trips_by_dir = {d: 0 for d in directions}
-            min_start_time_by_dir = {d: np.nan for d in directions}
-            max_end_time_by_dir = {d: np.nan for d in directions}  
-            mean_daily_duration_by_dir = {d: 0 for d in directions}
-            mean_daily_distance_by_dir = {d: 0 for d in directions}
-            group_by_dir = {d: None for d in directions}          
-            for d in directions:
-                group_by_dir[d] = group[group['direction_id'] == d]
-                g = group_by_dir[d]
-                for date in dates:
-                    stimes = g[g[date] > 0]['start_time'].values
-                    stimes = sorted([stime for stime in stimes 
-                      if 7*3600 <= stime <= 19*3600])
-                    headways_by_dir[d].extend(
-                      [stimes[i + 1] - stimes[i] 
-                      for i in range(len(stimes) - 1)])
-                if headways_by_dir[d]:
-                    max_headway_by_dir[d] = np.max(
-                      headways_by_dir[d])
-                    mean_headway_by_dir[d] = round(np.mean(
-                      headways_by_dir[d]))
-                mean_daily_num_trips_by_dir[d] = g['weight'].sum()
-                min_start_time_by_dir[d] = g['start_time'].min()
-                max_end_time_by_dir[d] = g['end_time'].max()
-                mean_daily_duration_by_dir[d] =\
-                  (g['duration']*g['weight']).sum()
-                mean_daily_distance_by_dir[d] =\
-                  (g['distance']*g['weight']).sum()
-            df = pd.DataFrame([[
-              d,
-              min_start_time_by_dir[d], 
-              max_end_time_by_dir[d], 
-              mean_daily_num_trips_by_dir[d], 
-              max_headway_by_dir[d], 
-              mean_headway_by_dir[d], 
-              mean_daily_duration_by_dir[d], 
-              mean_daily_distance_by_dir[d],
-              ] for d in directions], 
-              columns=[
-              'direction_id',
-              'min_start_time', 
-              'max_end_time', 
-              'mean_daily_num_trips', 
-              'max_headway', 
-              'mean_headway', 
-              'mean_daily_duration', 
-              'mean_daily_distance',
-              ])
-            df.index.name = 'foo'
-            return df
-
         if split_directions:
-            result = trips_stats.groupby('route_id').apply(
-              get_route_stats_by_direction).reset_index()
+            g = trips_stats.groupby(['route_id', 'direction_id'])
         else:
-            result = trips_stats.groupby('route_id').apply(
-              get_route_stats).reset_index()
+            g = trips_stats.groupby(['route_id'])
+
+        result = g.apply(get_route_stats).reset_index()
         del result['foo']
 
+        # Add speed column
+        result['mean_daily_speed'] = result['mean_daily_distance'].\
+          divide(result['mean_daily_duration'])
+
         # Convert route start times to time strings
         result['min_start_time'] = result['min_start_time'].map(lambda x: 
           utils.seconds_to_timestr(x))
 
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished routes stats in %.2f min' % minutes)    
-
         return result
 
-    def get_routes_time_series(self, trips_stats, dates):
+    def get_routes_time_series(self, trips_stats, dates, split_directions=True,
+      freq='5Min'):
         """
         Given ``trips_stats``, which is the output of 
-        ``self.get_trips_stats()``, use it to calculate the following 
-        four time series of routes stats:
+        ``self.get_trips_stats()``, return a time series version of the 
+        following route stats for the given dates:
         
         - mean daily number of vehicles in service by route ID
         - mean daily number of trip starts by route ID
-        - mean daily service duration (seconds) by route ID
-        - mean daily service distance (meters) by route ID
+        - mean daily service duration in hours by route ID
+        - mean daily service distance in kilometers by route ID
+        - mean daily speed in kilometers per hour
 
-        Each time series is a Pandas data frame over a 24-hour 
-        period with minute (period index) frequency (00:00 to 23:59).
+        The time series is a Pandas data frame with a period index 
+        for a 24-hour period sampled at the given frequency.
+        The maximum allowable frequency is 1 minute.
+        If multiples dates are given, a generic placeholder date of
+        2001-01-01 is used as the date for the period index.
+        Otherwise, the given date is used.
+
+        The columns of the data frame are hierarchical (multi-index) with
+
+        - top level: name = 'statistic', values = ['mean_daily_distance',
+          'mean_daily_duration', 'mean_daily_num_trip_starts', 
+          'mean_daily_num_vehicles', 'mean_daily_speed']
+        - middle level: name = 'stop_id', values = the active stop IDs
+        - bottom level: name = 'direction_id', values = 0s and 1s
+
+        If ``split_directions == False``, then don't include the bottom level.
         
-        Return the time series as values of a dictionary with keys
-        'mean_daily_num_vehicles', 'mean_daily_num_trip_starts', 
-        'mean_daily_duration', 'mean_daily_distance'.
-
         NOTES:
 
         - To resample the resulting time series use the following methods:
             - for 'mean_daily_num_vehicles' series, use ``how=np.mean``
             - for the other series, use ``how=np.sum`` 
-        - To remove the placeholder date (2001-1-1) and seconds from any 
-          of the time series f, do ``f.index = [t.time().strftime('%H:%M') 
+            - 'mean_daily_speed' can't be resampled and must be recalculated
+              from 'mean_daily_distance' and 'mean_daily_duration' 
+        - To remove the placeholder date (2001-1-1) and seconds from the 
+          time series f, do ``f.index = [t.time().strftime('%H:%M') 
           for t in f.index.to_datetime()]``
-        - Takes about 1.5 minutes on the SEQ feed.
+        - Takes about 0.6 minutes on the Portland feed given the first
+          five weekdays of the feed.
         """  
         if not dates:
             return 
-
-        t1 = dt.datetime.now()
-        stats = trips_stats
-        routes = sorted(self.routes['route_id'].values)
-        print(t1, 'Creating routes time series for {!s} routes...'.format(
-          len(routes)))
-
-        # Merge trips_stas with trips activity, get trip weights,
+        # Merge trips_stats with trips activity, get trip weights,
         # and drop 0-weight trips
         n = len(dates)
-        stats = pd.merge(stats, self.get_trips_activity(dates))
+        stats = pd.merge(trips_stats, self.get_trips_activity(dates))
         stats['weight'] = stats[dates].sum(axis=1)/n
         stats = stats[stats.weight > 0]
         num_trips = stats.shape[0]
-        
+
+        if split_directions:
+            # Separate route IDs by direction: <route ID>-0 and <route ID>-1
+            stats['route_id'] = stats['route_id'] + '-' +\
+              stats['direction_id'].map(str)
+            
+        routes = sorted(stats['route_id'].unique())
         # Initialize time series
         if n > 1:
             # Assign a uniform generic date for the index
@@ -1059,9 +886,12 @@ class Feed(object):
         day_start = pd.to_datetime(date_str + ' 00:00:00')
         day_end = pd.to_datetime(date_str + ' 23:59:00')
         rng = pd.period_range(day_start, day_end, freq='Min')
-        names = ['mean_daily_num_vehicles', 
-          'mean_daily_num_trip_starts', 'mean_daily_duration', 
-          'mean_daily_distance']
+        names = [
+          'mean_daily_num_vehicles', 
+          'mean_daily_num_trip_starts', 
+          'mean_daily_duration', 
+          'mean_daily_distance',
+          ]
         series_by_name = {}
         for name in names:
             series_by_name[name] = pd.DataFrame(np.nan, index=rng, 
@@ -1106,45 +936,16 @@ class Feed(object):
                       weight, index=g.index), fill_value=0)
                 elif name == 'mean_daily_duration':
                     f.loc[criterion, route] = g.add(pd.Series(
-                      weight*60, index=g.index), fill_value=0)
+                      weight/60, index=g.index), fill_value=0)
                 else:
                     # name == 'distance'
                     f.loc[criterion, route] = g.add(pd.Series(
                       weight*row['distance']/num_bins, index=g.index),
                       fill_value=0)
 
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished routes time series in %.2f min' % minutes)    
+        g = utils.combine_time_series(series_by_name, kind='route')
+        return utils.downsample(g, freq=freq)
 
-        return series_by_name
-
-    # def get_route_timetable(self, route, date):
-    #     """
-    #     Given a route (route_id) and a date (datetime.date object),
-    #     return a Pandas data frame describing the time table for the
-    #     given stop on the given date with the columns
-
-    #     - arrival_time
-    #     - departure_time
-    #     - trip_id
-
-    #     and ordered by arrival time.
-    #     """
-    #     # Get active trips and merge with stop times
-    #     trips_activity = self.get_trips_activity([date])
-    #     ta = trips_activity[trips_activity[date] > 0]
-    #     stop_times = self.stop_times
-    #     f = pd.merge(ta, stop_times)
-        
-    #     def get_first_trip(group):
-    #         g = group.sort('departure_time')
-    #         return g[['arrival_time', 'departure_time']].iloc[0]
-
-    #     return f[f['route_id'] == route].groupby('trip_id').apply(
-    #       get_first_trip).reset_index()
-
-    # TODO: test more and improve readme
     def dump_all_stats(self, directory, dates=None, freq='1H'):
         """
         Into the given directory, dump to separate CSV files the outputs of
@@ -1160,18 +961,10 @@ class Feed(object):
         on units and include some useful charts.
 
         If no dates are given, then use ``self.get_first_week()[:5]``.
-
-        NOTES:
-
-        Takes about 15 minutes on the SEQ feed.
         """
         import os
         import textwrap
 
-        # Time function call
-        t1 = dt.datetime.now()
-        print(t1, 'Beginning process...')
-
         if not os.path.exists(directory):
             os.makedirs(directory)
         if dates is None:
@@ -1183,8 +976,8 @@ class Feed(object):
         readme = """
         Notes 
         =====
-        - Distances are measured in meters and durations are measured in
-        seconds
+        - Distances are measured in kilommeters and durations are measured in
+        hours
         - Stats were calculated for the period {!s}
         """.format(dates_str)
         
@@ -1197,14 +990,8 @@ class Feed(object):
 
         # Stops time series
         sts = self.get_stops_time_series(dates)
-        sts = utils.downsample_stops_time_series(sts, freq=freq)
-        for name, f in sts.items():
-            # Remove date from timestamps
-            g = f.copy()
-            g.index = [d.time() for d in g.index.to_datetime()]
-            g.T.to_csv(directory + 'stops_time_series_%s_%s.csv' %\
-              (name, freq), index_label='stop_id')
-
+        sts = utils.downsample(sts, freq=freq)
+        sts.to_csv(directory + 'stops_time_series_{!s}.csv'.format(freq))
 
         # Trips stats
         trips_stats = self.get_trips_stats()
@@ -1216,19 +1003,10 @@ class Feed(object):
 
         # Routes time series
         rts = self.get_routes_time_series(trips_stats, dates)
-        rts = utils.downsample_routes_time_series(rts, freq=freq)
-        for name, f in rts.items():
-            # Remove date from timestamps
-            g = f.copy()
-            g.index = [d.time() for d in g.index.to_datetime()]
-            g.T.to_csv(directory + 'routes_time_series_%s_%s.csv' %\
-              (name, freq), index_label='route_id')
+        rts = utils.downsample(rts, freq=freq)
+        rts.to_csv(directory + 'routes_time_series_{!s}.csv'.format(freq))
 
         # Plot sum of routes stats 
         fig = utils.plot_routes_time_series(rts)
         fig.tight_layout()
-        fig.savefig(directory + 'routes_time_series_agg.pdf', dpi=200)
-        
-        t2 = dt.datetime.now()
-        minutes = (t2 - t1).seconds/60
-        print(t2, 'Finished process in %.2f min' % minutes)    
\ No newline at end of file
+        fig.savefig(directory + 'routes_time_series_agg.pdf', dpi=200)
\ No newline at end of file
