@@ -8,14 +8,13 @@ from shapely.geometry import Point, LineString, mapping
 from gtfs_toolkit.feed import *
 from gtfs_toolkit.utils import *
 
-# Test feeds:
-cairns = Feed('gtfs_toolkit/tests/cairns_20140223.zip')
-cairns_shapeless = Feed('gtfs_toolkit/tests/cairns_20140223.zip')
+# Load test feeds
+cairns = Feed('gtfs_toolkit/tests/cairns_gtfs.zip')
+cairns_shapeless = Feed('gtfs_toolkit/tests/cairns_gtfs.zip')
 cairns_shapeless.shapes = None
-portland = Feed('gtfs_toolkit/tests/portland_20140518.zip')
 
 class TestFeed(unittest.TestCase):
-
+    #"""
     def test_seconds_to_timestr(self):
         seconds = 3600 + 60 + 1
         timestr = '01:01:01'
@@ -41,6 +40,7 @@ class TestFeed(unittest.TestCase):
         self.assertEqual(get_segment_length(s, p), 0)
 
     def test_init(self):
+        portland = Feed('gtfs_toolkit/tests/portland_gtfs.zip')
         for feed in [cairns, portland]:
             self.assertIsInstance(feed.routes, pd.core.frame.DataFrame)
             self.assertIsInstance(feed.stops, pd.core.frame.DataFrame)
@@ -84,7 +84,7 @@ class TestFeed(unittest.TestCase):
         self.assertTrue(feed.is_active_trip(trip, date1))
         self.assertFalse(feed.is_active_trip(trip, date2))
 
-        feed = portland
+        feed = Feed('gtfs_toolkit/tests/portland_gtfs.zip')
         trip = '4526377'
         date1 = dt.date(2014, 5, 18)
         date2 = dt.date(2012, 5, 17)
@@ -172,11 +172,55 @@ class TestFeed(unittest.TestCase):
     def test_get_stops_time_series(self):
         feed = cairns
         dates = feed.get_first_week()
-        stops_ts = feed.get_stops_time_series(dates)['mean_daily_num_vehicles']
-        # Should have the correct shape
-        self.assertEqual(stops_ts.shape[0], 24*60)
-        self.assertEqual(stops_ts.shape[1], feed.stops.shape[0])
+        sa = feed.get_stops_activity(dates)
+        sa = sa[sa[dates].sum(axis=1) > 0]
+        for split_directions in [True, False]:
+            stops_ts = feed.get_stops_time_series(dates, freq='1H',
+              split_directions=split_directions) 
+            # Should be a data frame
+            self.assertIsInstance(stops_ts, pd.core.frame.DataFrame)
+            # Should have the correct shape
+            self.assertEqual(stops_ts.shape[0], 24)
+            if split_directions:
+                expect_num_cols = 2*sa.shape[0]
+            else:
+                expect_num_cols = sa.shape[0]
+            self.assertEqual(stops_ts.shape[1], expect_num_cols)
 
+    def test_get_routes_stats(self):
+        feed = cairns
+        dates = feed.get_first_week()
+        trips_stats = feed.get_trips_stats()
+        f = pd.merge(trips_stats, feed.get_trips_activity(dates))
+        f = f[f[dates].sum(axis=1) > 0]
+        for split_directions in [True, False]:
+            rs = feed.get_routes_stats(trips_stats, dates, 
+              split_directions=split_directions)
+            # Should be a data frame of the correct shape
+            self.assertIsInstance(rs, pd.core.frame.DataFrame)
+            if split_directions:
+                f['tmp'] = f['route_id'] + '-' +\
+                  f['direction_id'].map(str)
+            else:
+                f['tmp'] = f['route_id'].copy()
+            expect_num_routes = len(f['tmp'].unique())
+            self.assertEqual(rs.shape[0], expect_num_routes)
+    
+    def test_get_routes_time_series(self):
+        feed = cairns 
+        dates = feed.get_first_week()
+        trips_stats = feed.get_trips_stats()
+        for split_directions in [True, False]:
+            f = feed.get_routes_stats(trips_stats, dates, 
+              split_directions=split_directions)
+            rts = feed.get_routes_time_series(trips_stats, dates, 
+              split_directions=split_directions, freq='1H')
+            # Should be a data frame of the correct shape
+            self.assertIsInstance(rts, pd.core.frame.DataFrame)
+            expect_num_routes = 5*f.shape[0]
+            self.assertEqual(rts.shape[0], 24)
+            self.assertEqual(rts.shape[1], expect_num_routes)
+    #"""
 
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
